/**
 * TopoSurface - by Matt Ward
 * This program reads in a file of elevation data and displays
 * a rubber sheet view.  Interactions supported include zooming,
 * panning, and rotation.  Panning is via mouse motions, while
 * zooming and rotation are keyboard clicks.
 */
int[][] distances = new int[400][400]; // buffer for elevation data
int shiftX, shiftY; // translation amounts
float angleX, angleY, angleZ; // rotation amounts
int camera_distance; // zooming amount

void setup() {
  size(400,400, P3D); // elevation data is 400 by 400
  shiftX = width/2; // center the initial translation
  shiftY = height/2;
  angleX = 0;
  angleY = 0;
  angleZ = 0;
  camera_distance = 500;
   	
// open a file and read its binary data 
byte b[] = loadBytes("colorado_elev.vit"); 

// skip 268 byte header and convert bytes to ints
  for(int i = 0; i < 400; i++)
   for(int j = 0;j < 400; j++)
      distances[i][j] = b[268 + i*400 + j] & 0xff;    
}

void draw() {
  background(0);
  lights();
// set camera to look at middle of data
  camera(200, 200, camera_distance,
         200, 200, 0,
         0.0, 1.0, 0.0); 
// interpret left mouse clicks as translates
  if(mousePressed && mouseButton == LEFT)  {
    shiftX = mouseX - width/2;
    shiftY = mouseY - height/2;
  }
  translate(shiftX, shiftY);
// now perform all rotations
  rotateY(angleY);
  rotateX(angleX);
  rotateZ(angleZ);
// don't draw edges, set fill to white, and draw the surface
  noStroke();
  fill(255, 255, 255); 
  drawSurface(distances); 
}

// handle all keystroke events for zoom and rotate (fixed amounts)
  void keyPressed()  {
    if(key == '-') camera_distance += 100;
    else  if (key == '+') camera_distance -= 100;
    else if(key == 'x') angleX += .25;
    else if(key == 'X') angleX -= .25;
    else if(key == 'y') angleY += .25;
    else if(key == 'Y') angleY -= .25;
    else if(key == 'z') angleZ += .25;
    else if(key == 'Z') angleZ -= .25;
  }
 
// to draw the surface, make a bunch of triangle strips 
void drawSurface(int distances[][])  {

int x = 1; // x and y are the distances between adjacent data points
int y = 1;
float px = -width/2, py = -height/2; // we want things centered on origin
float pts = 399; // the number of triangles in a strip

for(int j = 0;j < pts;j++)  {
// create a triangle strip for each row in the data
  beginShape(TRIANGLE_STRIP); 
// first vertices are along edge
    vertex(px,py, (float)distances[j][0]);
    vertex(px,py+y, (float)distances[j+1][0]);
// for rest of vertices, alternate between 2 rows
    for (int i = 0; i < pts; i++) {
      px = px + x;
      vertex(px, py, (float)distances[j][i+1]);
      vertex(px, py+y, (float)distances[j+1][i+1]); 
      }
  endShape();
// shift row and reset x to edge
  py = py + y;
  px = -width/2;
  }
}
